UPET_AVAILABLE_MODELS = [
    "pet-mad-s",
    "pet-omat-xs",
    "pet-omat-s",
    "pet-omat-m",
    "pet-omat-l",
    "pet-omat-xl",
    "pet-oam-l",
    "pet-oam-xl",
    "pet-omad-xs",
    "pet-omad-s",
    "pet-omad-l",
    "pet-omatpes-l",
    "pet-spice-s",
    "pet-spice-l",
]

UPET_NO_NC_SUPPORT_MODELS = [
    "pet-mad-s-v1.0.2",
    "pet-spice-s-v0.2.0",
    "pet-spice-l-v0.2.0",
]

UPET_UQ_SUPPORTED_MODELS = [
    "pet-mad-s-v1.0.2",
]

# PET-MAD DOS
PET_MAD_DOS_LATEST_STABLE_VERSION = "1.0"
PET_MAD_DOS_AVAILABLE_VERSIONS = ["1.0"]
